/**
 * lcd-clock by peter dannegger
 *
 * via shift register 74HC164
 *
 * connections:
 *    avr      -> lcd-driver (SPI)
 *    74HC164  -> lcd:
 *    ----------------
 *    3     ->    D4
 *    4     ->    D7
 *    5     ->    D6
 *    6     ->    D5
 *    10    ->    D1
 *    11    ->    D2
 *    12    ->    D3
 *    13    ->    D0
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "config.h"
#include "lcd.h"


static void lcdOut(uint8_t d, uint8_t isData) {
   uint8_t  i;

   PORT_COMM_SYNC &= (uint8_t) ~(1 << PIN_COMM_SYNC);
   for (i=0; i < 8; i++) {
      PORT_COMM_CLOCK &= (uint8_t) ~(1 << PIN_COMM_CLOCK);
      if (d & BitNo[i]) PORT_COMM_DATA |= 1 << PIN_COMM_DATA;
      else              PORT_COMM_DATA &= (uint8_t) ~(1 << PIN_COMM_DATA);
      PORT_COMM_CLOCK |= 1 << PIN_COMM_CLOCK;
   }
   if (isData) PORT_COMM_DATA |= 1 << PIN_COMM_DATA;
   else        PORT_COMM_DATA &= (uint8_t) ~(1 << PIN_COMM_DATA);
   PORT_COMM_SYNC |= 1 << PIN_COMM_SYNC;
   asm volatile ("nop");
   PORT_COMM_SYNC &= (uint8_t) ~(1 << PIN_COMM_SYNC);
   _delay_us(46);
}


void lcdCommand(uint8_t c) {
   lcdOut(c, 0);

   if (c & 0x80) lcdPos = c;
   switch (c) {
   case 1:
   case 2:
   case 3:
      _delay_ms(1.6);
   }
}


void lcdData(uint8_t d) {
   lcdOut(d, 1);
   if (++lcdPos == 0x90)    lcdPos = 0xC0;
   else if (lcdPos == 0xD0) lcdPos = 0x80;
   else return;

   lcdCommand(lcdPos);
}


void lcdInit(void) {
   _delay_ms(15);
   lcdCommand(0x30);
   _delay_ms(4.1);
   lcdCommand(0x30);
   _delay_us(100);
   lcdCommand(0x30);

   lcdCommand(0x38);                                       // 2 lines 5*7
   lcdCommand(0x08);                                       // display off
   lcdCommand(0x01);                                       // display clear
   lcdCommand(0x06);
   lcdCommand(0x0C);                                       // no cursor no blink
   lcdPos = 0x80;
}
